% Network Component Mapping Algorithm - NCM sampling algorithm described in 
%   Brynildsen MP, Wu TY, Jang SS, JC Liao "Biological Network Mapping and Source Signal Deduction"
%   
% 
% function [r_net,run_zero,U,S,V,flag] = NCM(data,regulators,path_threshold,initial_threshold) 
%    
%     Parameters
%     ----------
%       data: gene expression matrix (genes x experiments)
%       regulators: number of transcription factors
%       path_threshold: contribution threshold for recursive algorithm
%       initial_threshold: contribution threshold for initial mapping
%       type: 1 skip initial mapping criterion 2, 0 do not skip initial mapping criterion 2
%
%     Output
%     ----------
%       r_net: NCM deduced network
%       run_zero: # of zeros in r_net
%       U,S,V: rank reduced singular value decomposition
%       flag: 1 designates that recursive algorithm was successful, 0 designates that the recursive algorithm was unsuccessful
%       
%        
%    
% ------------------------------------------------------------------------------------------
% Last revision March 6th, 2007 via MP Brynildsen, UCLA
% ------------------------------------------------------------------------------------------


function [r_net,run_zero,r_maxxes,U,S,V,flag] = NCMalgorithm(data,regulators,path_threshold,initial_threshold,type,path_number)

%Initialize variables
[N,M] = size(data);
L = regulators;

%Randomize start position of random number generators
rand('state',sum(100*clock))
randn('state',sum(100*clock))

%Initialize a full network
real_net = rand(N,L) + ones(N,L);

%Generate the initial versatile network
first = 0;
count = 1;
while (first == 0),
    net = randn(N,L);
    for kk = 1:L,   %Seed L-1 zeros per column
        vector = sortrows([net(:,kk) [1:N]']);
        zeros = vector(1:(L-1),2);
        net(zeros,kk) = 0;
    end
    r = rankcheckmark(net);
    if (r == L*(L-1)),  %Check NCA-compliance
        [a,p,condX] = NetMap(data,net,initial_threshold); %null space calculation
        if (condX < L & cond(p) < L*(L-1) & cond(a) < L*(L-1))  %Check the condition number of X and p, L and L*(L-1) are empirically derived
            x = find(abs(a) == 0);
            z = find(abs(net) == 0);
            [xx,xxx] = size(x);
            r = rankcheckmark(a); %Check NCA-compliance
            [zero_un,ique] = size(unique([x;z]));   
            zero_per_column = 1;
            %Strict/Relaxed Constraint
            if (type == 1), %Relaxed, initial mapping generates 
                if (size(find(a == 0),1) <= L*(L-1))
                    zero_per_column = 0;
                end
            elseif (type == 0), %Strict, initial mapping generates at least +1 zero/column (L or more zeroes/column)
                for mb = 1:L,
                    col_z = size(find(a(:,mb) == 0));
                    if (col_z(1) == (L-1)),
                        zero_per_column = 0;
                    end
                end
            else
                fprintf('Initial Mapping Criterion neglected\n');
            end
            if (zero_per_column == 1 & r == L*(L-1) & zero_un == xx),  %Checks NCA-compliance, every column has a behavior space constraint, and that the original versatile network is a parent to the current one
                first = 1;
                real_net = a;
            end
        end
    end
end

pre_net = real_net;
pre_net_zeros = size(find(pre_net == 0),1);
%fprintf('Zeros from Initial Mapping: %.0f\nBegin Fine Mapping\n',pre_net_zeros);
%Perform SVD
[U,S,V] = svd(data);
U(:,(L+1):N) = [];
S(:,(L+1):M) = [];
S((L+1):N,:) = [];
V(:,(L+1):M) = [];
flag = 0;
% Fine Mapping
for runs = 1:path_number,
    run_net(:,runs) = reshape(pre_net,N*L,1);
    run_zero(runs,:) = size(find(pre_net == 0),1);
    stability_skip = 1;
    real_net = pre_net;
    real_net(find(real_net ~= 0)) = 1;
    real_net = rand(size(real_net)).*real_net;% randomize the entries of the network, because we will use the maximum value entry for probing
    for hh = 1:L,
        place = find(real_net(:,hh) == max(real_net(:,hh)));
        maxxes(hh) = place;
    end
    run_maxxes(runs,:) = maxxes;
    for good_counter = 1:L, %Perform L times
        safety_net = real_net;
        safety_net_zeros = size(find(safety_net == 0),1);
        [X] = NCASVD(safety_net,U,S,V);
        for kk = 1:L, %Probe Columns
            next_net = safety_net;
            non_zeros = find(safety_net(:,kk) ~= 0);
            [nonz,ero] = size(non_zeros);
            partial_net = [1:(kk-1),(kk+1):L];
            bad_rows = find(sum(safety_net(:,partial_net)') == 0);
            to_non_versatilize = [safety_net(:,kk) [1:N]'];
            to_non_versatilize(bad_rows,:) = [];
            to_non_versatilize(find(to_non_versatilize(:,1) == 0),:) = [];
            to_non_versatilize(:,1) = [];
            [to,nv] = size(to_non_versatilize);
            for pp = 1:to, %Toggle through non-zero entries setting each to zero
                net = safety_net;
                net(to_non_versatilize(pp),kk) = 0;
                if (rankcheckmark(net) == L*(L-1)), %NCA-compliance of probe net
                    %-----Set the Path
                    for nn = 1:L,
                        net(maxxes(nn),nn) = max(net(:,nn)) + 1;
                    end
                    %-----Map
                    [x_kk] = NCASVD_single(net,U,S,V,kk);
                    Xtemp = X;
                    Xtemp(:,kk) = x_kk;
                    if (cond(Xtemp) < L),
                        a = U*Xtemp;
                        p = inv(Xtemp)*S*V';
                        %-----Calculate Contribution by row
                        for kj = 1:N,
                            for jk = 1:L,
                                contribution(kj,jk) = sum((a(kj,jk)*p(jk,:)).*(a(kj,jk)*p(jk,:)))^.5;
                            end
                            contribution(kj,:) = contribution(kj,:)/sum(contribution(kj,:));
                        end
                        %-----Trimming
                        a(find(contribution < path_threshold)) = 0; 
                        contribution = [];
                        if (rankcheckmark(a) == L*(L-1)), %NCA-compliance of resultant net
                            [a,p] = ncanormalize(a,p);
                            x = find(abs(a) == 0); [xx,xxx] = size(x);
                            y = find(next_net == 0); [yy,yyy] = size(y);
                            z = find(safety_net == 0); [zz,zzz] = size(z);
                            [zero_un,ique] = size(unique([x;z]));
                            if (xx > yy & cond(p) < L*(L-1) & cond(a) < L*(L-1) & a(to_non_versatilize(pp),kk) == 0 & zero_un == xx & a(maxxes(kk),kk) ~= 0),
                                kosher = 1;
                                for kt = 1:L,
                                    if (a(maxxes(kt),kt) == 0),
                                        kosher = 0;
                                    end
                                end
                                if (kosher == 1),
                                    next_net = a;
                                    stability_skip = 0; %Send to Stability Check
                                end
                            end
                        end
                    end
                end
            end
            %----Catalgoue for choice
            choice_net(:,kk) = reshape(next_net,N*L,1);
            choice(kk,:) = size(find(next_net == 0));
        end
        chosen = find(choice(:,1) == max(choice(:,1))); %Choose largest number of zeros
        [cho,sen] = size(chosen);
        if (cho == L & choice(1) == safety_net_zeros),
            break
        end
        %-----In lieu of a Tie
        chosen(:,2) = randn(size(chosen)); 
        chosen = chosen(find(chosen(:,2) == max(chosen(:,2))),1);
        %-----Redefine parent net
        real_net = reshape(choice_net(:,chosen),N,L);
        choice = [];
    end
    if (size(find(real_net == 0),1) > pre_net_zeros),
        %fprintf('Step %.0f, Zeros: %.0f\n',runs,size(find(real_net == 0),1));
    end
    if (stability_skip == 0),
        %Stability Check
        before = size(find(real_net == 0),1);
        for nn = 1:L,
            real_net(maxxes(nn),nn) = max(real_net(:,nn)) + 1;
        end
        after = size(find(real_net == 0),1);
        if (before ~= after),
            fprintf('Problem with maxxes\n');
            stop
        end
        [X] = NCASVD(real_net,U,S,V);
        a = U*X;
        p = inv(X)*S*V';
        [a,p] = ncanormalize(a,p);
        for kj = 1:N,
            for jk = 1:L,
                contribution(kj,jk) = sum((a(kj,jk)*p(jk,:)).*(a(kj,jk)*p(jk,:)))^.5;
            end
            contribution(kj,:) = contribution(kj,:)/sum(contribution(kj,:));
        end
        a(find(contribution < path_threshold)) = 0;
        contribution = [];
        x = find(a == 0);
        y = find(real_net == 0);
        if (cond(X) < L & cond(p) < L*(L-1) & cond(a) < L*(L-1) & rankcheckmark(a) == L*(L-1) & size(unique([x;y]),1) == size(y,1) & size(y,1) == size(x,1)),
            run_net(:,runs) = reshape(a,N*L,1);
            run_zero(runs,:) = size(find(a == 0),1);
            run_maxxes(runs,:) = maxxes;
            if (min(sum(abs(a))) == 0)
                a,p,cond(X),cond(p),cond(a),rankcheckmark(a)
                stop
            end
        else
            %fprintf('Unstable Solution: Run:%.0f, condX:%.2f, condp:%.2f, conda:%.2f, nca:%.0f, old zeros:%.0f, new zeros:%.0f\n', runs, cond(X), cond(p), cond(a), rankcheckmark(a), size(y,1), size(x,1));
        end
    end
end
%fprintf('Finished Fine Mapping\n');
%ready data for outputting
x = find(run_zero == max(run_zero(:,1)));
r_maxxes = run_maxxes(x(1),:);
r_net = reshape(run_net(:,x(1)),N,L);
NumberofZeros = size(find(r_net == 0));
fprintf('Network conatains %.0f zeros\n\n',NumberofZeros(1));
if (NumberofZeros(1) > pre_net_zeros),
    flag = 1;
end



%---zscore---
function [zdata] = zscore_alter(E)
[N,M] = size(E);
Choice1 = 'By Row';
Choice2 = 'By Column';
either(1) = cellstr(Choice1);
either(2) = cellstr(Choice2);
Selection = 1;
if (Selection == 1),%zscore by row
    for kk = 1:N,
        Mean = mean(E(kk,:));
        Std = std(E(kk,:));
        for jj = 1:M,
            zdata(kk,jj) = (E(kk,jj) - Mean)/Std;
        end
    end
end


%---rankcheckmark---
function R= rankcheckmark(alpha)
[N,L]= size(alpha);
R= 0;   % --> initialize rank of the augmented sparse system
nz= 0;  % --> initialize total # of zeros in the augmented system
for l= 1:L,
    v= find(alpha(:,l)==0);
    nz= nz+length(v);
    R= R+ rank(alpha(v,:));
end


%---NetMap---
function  [nA,nP,condX] = NetMap(E,A,initial_threshold);
[N,L] = size(A);
[N,M] = size(E);
[U,S,V] = svd(E);
U(:,(L+1):N) = [];
S(:,(L+1):M) = [];
S((L+1):N,:) = [];
V(:,(L+1):M) = [];
countrow = 1;
countcolumn = 1;
for i = 1:L,
    Ularger(countrow:(countrow + N-1),countcolumn:(countcolumn + L-1)) = U;
    countcolumn = countcolumn + L;
    countrow = countrow + N;
end
Avector = reshape(A,N*L,1);
nonzero = find(Avector ~= 0);
Avector(nonzero(:),:) = [];
Ularger(nonzero(:),:) = [];
NullSpace = null(Ularger);
condX = 20;
refX = 0;
if (size(NullSpace,2) ~= L),
    A
    size(Ularger)
    Avector
    stop
end
for jj = 1:L,
    nX = reshape(NullSpace(:,jj),L,L);
    if(condX > cond(nX));
        refX = jj;
        condX = cond(nX);
    end
end
if (refX ~= 0),
    nX = reshape(NullSpace(:,refX),L,L);
    nA = U*nX;
    nP = inv(nX)*S*V';
    for kj = 1:N,
        for jk = 1:L,
            contribution(kj,jk) = sum((nA(kj,jk)*nP(jk,:)).*(nA(kj,jk)*nP(jk,:)))^.5;
        end
        contribution(kj,:) = contribution(kj,:)/sum(contribution(kj,:));
    end
    nA(find(contribution < initial_threshold)) = 0;
    contribution = [];
    [nA,nP] = ncanormalize(nA,nP);
else
    nX = 0;
    nA = 0;
    nP = 0;
end


%Normalization%
function [A1,P1] = ncanormalize(A,P);
[N,L] = size(A);
Z = 0;
NZ = 0;
Sum = 0;
Zero = [];
NonZero = [];
ColumnSum = [];
NF = [];
for j = 1:L,
    for i = 1:N,
        if (A(i,j) == 0),
            Z = Z + 1;
        else
            NZ = NZ + 1;
            Sum = Sum + abs(A(i,j));
        end
    end
    NonZero(j,j) = NZ;
    Zero(j,j) = Z;
    ColumnSum(j,j) = Sum;
    NF(j,j) = ColumnSum(j,j)/NonZero(j,j);
    NZ = 0;
    Z = 0;
    Sum = 0;
end
A1 = A*inv(NF);
P1 = NF*P;


%---NCASVD---
function [X] = NCASVD(A,U,S,V);
[N,L] = size(A);
[M] = size(V,1);
countrow = 1;
countcolumn = 1;
for i = 1:L,
    Ularger(countrow:(countrow + N-1),countcolumn:(countcolumn + L-1)) = U;
    countcolumn = countcolumn + L;
    countrow = countrow + N;
end
Maxx = max(abs(A));
Avector = reshape(A,N*L,1);
count = 1;
for j = 1:N*L,
    if (Avector(j)~=0)
        G = find(Maxx == abs(Avector(j))); %select max number for rep...
        [gg,hh] = size(G);
        if (hh == 0)
            X(count) = j;
            count  = count + 1;
        end
    end
end
Avector(X(:),:) = [];
Ularger(X(:),:) = []; 
P = find(Avector ~=0);
Avector(P(:)) = 1;
x = Ularger\Avector;
X = reshape(x,L,L);


%---NCASVD_single---
function [x_kk] = NCASVD_single(A,U,S,V,kk);
A = abs(A);
[N,L] = size(A);
[M] = size(V,1);
Avector = A(:,kk); % The kkth column
x = find(Avector == 0); xx = find(Avector == max(Avector)); X = [x;xx];
Avector = Avector(X(:),:);
U = U(X(:),:);
Avector(find(Avector ~=0)) = 1;
x_kk = U\Avector;
if (sum(x_kk) == 0),
    A
    U
    Avector
    x_kk
    x
    xx
end