% Mastercode for Network Component Mapping Algorithm - NCM sampling algorithm described in 
%   Brynildsen MP, Wu TY, Jang SS, JC Liao "Biological Network Mapping and Source Signal Deduction"
%   
% 
% function [SaveIt] = multipleRuns_NCM(data,regulators,initial_threshold,path_threshold,type,path_number,iteration_number)
%    
%     Parameters
%     ----------
%       data: gene expression matrix (genes x experiments)
%       regulators: number of transcription factors
%       path_threshold: contribution threshold for recursive algorithm
%       initial_threshold: contribution threshold for initial mapping
%       type: 0 impose initial mapping crterion 1, 1 relax initial mapping criterion 1, other skip initial 
%       mapping criterion 1
%       
%       
%
%     Output
%     ----------
%       SaveIt: log of NCM deduced networks from iterations of NCM
%       
%        
%    
% ------------------------------------------------------------------------------------------
% Last revision March 6th, 2007 via MP Brynildsen, UCLA
% -------------------------------------------------------------------------
% -----------------


function [SaveIt] = multipleRuns_NCM(data,regulators,initial_threshold,path_threshold,type,path_number,iteration_number)

%z-score data
[data] = zscore_alter(data);
[N,M] = size(data);
[L] = regulators;
zNumber = 0;
SaveIt = zeros(N*L+1,100);

% NCM iterations
for kk = 1:iteration_number,
    fprintf('Iteration: %.0f\n',kk);
    [net,run_zero,r_maxxes,U,S,V,flag] = NCMalgorithm(data,regulators,path_threshold,initial_threshold,type,path_number);
    if (flag == 1), %Fine Mapping Identified Pattern
        before = size(find(net == 0),1);
        for nn = 1:L,
            net(r_maxxes(nn),nn) = max(net(:,nn)) + 1;
        end
        after = size(find(net == 0),1);
        if (before ~= after),
            fprintf('Problem with before,after\n');
            stop
        end
        [X] = NCASVD(net,U,S,V);
        a = U*X;
        p = inv(X)*S*V';
        for kj = 1:N,
            for jk = 1:regulators,
                contribution(kj,jk) = sum((a(kj,jk)*p(jk,:)).*(a(kj,jk)*p(jk,:)))^.5;
            end
            contribution(kj,:) = contribution(kj,:)/sum(contribution(kj,:));
        end
        a(find(contribution < path_threshold)) = 0;
        contribution = [];
        e = a*p;
        [norm] = frobnorm(data,e);
        SaveIt(1:N*L,kk) = reshape(a,N*L,1);
        SaveIt(N*L+1,kk) = norm;
    end
    if (flag == 0), %Fine Mapping Did Not Identify Pattern
        [X] = NCASVD(net,U,S,V);
        a = U*X;
        p = inv(X)*S*V';
        for kj = 1:N,
            for jk = 1:regulators,
                contribution(kj,jk) = sum((a(kj,jk)*p(jk,:)).*(a(kj,jk)*p(jk,:)))^.5;
            end
            contribution(kj,:) = contribution(kj,:)/sum(contribution(kj,:));
        end
        a(find(contribution < initial_threshold)) = 0;
        contribution = [];
        e = a*p;
        [norm] = frobnorm(data,e);
        SaveIt(1:N*L,kk) = reshape(a,N*L,1);
        SaveIt(N*L+1,kk) = norm;
    end
    save SaveIt SaveIt -ascii
end









%---zscore---
function [zdata] = zscore_alter(E)
[N,M] = size(E);
Choice1 = 'By Row';
Choice2 = 'By Column';
either(1) = cellstr(Choice1);
either(2) = cellstr(Choice2);
Selection = 1;
if (Selection == 1),%zscore by row
    for kk = 1:N,
        Mean = mean(E(kk,:));
        Std = std(E(kk,:));
        for jj = 1:M,
            zdata(kk,jj) = (E(kk,jj) - Mean)/Std;
        end
    end
end



%Normalization%
function [A1,P1] = ncanormalize(A,P);
[N,L] = size(A);
Z = 0;
NZ = 0;
Sum = 0;
Zero = [];
NonZero = [];
ColumnSum = [];
NF = [];
for j = 1:L,
    for i = 1:N,
        if (A(i,j) == 0),
            Z = Z + 1;
        else
            NZ = NZ + 1;
            Sum = Sum + abs(A(i,j));
        end
    end
    NonZero(j,j) = NZ;
    Zero(j,j) = Z;
    ColumnSum(j,j) = Sum;
    NF(j,j) = ColumnSum(j,j)/NonZero(j,j);
    NZ = 0;
    Z = 0;
    Sum = 0;
end
A1 = A*inv(NF);
P1 = NF*P;


%FrobNorm
function [F] = frobnorm(E,E1);
% This function caluculates the frombeus norm of the entire data set (E)
%[N,L] = size(A);
[N,M] = size(E);
Sq = 0;
Sum = 0;
AP = [];
AP = E1;
for i = 1:N,
    for j = 1:M,
        Sq = (E(i,j)-AP(i,j))^2;
        Sum = Sum + Sq;
    end
end
F = (Sum)^(1/2);


%---NCASVD---
function [X] = NCASVD(A,U,S,V);
[N,L] = size(A);
[M] = size(V,1);
countrow = 1;
countcolumn = 1;
for i = 1:L,
    Ularger(countrow:(countrow + N-1),countcolumn:(countcolumn + L-1)) = U;
    countcolumn = countcolumn + L;
    countrow = countrow + N;
end
Maxx = max(abs(A));
Avector = reshape(A,N*L,1);
count = 1;
for j = 1:N*L,
    if (Avector(j)~=0)
        G = find(Maxx == abs(Avector(j))); %select max number for rep...
        [gg,hh] = size(G);
        if (hh == 0)
            X(count) = j;
            count  = count + 1;
        end
    end
end
Avector(X(:),:) = [];
Ularger(X(:),:) = [];
P = find(Avector ~=0);
Avector(P(:)) = 1;
x = Ularger\Avector;
X = reshape(x,L,L);